/* infolister-about.js
 * Implements the "about:info" hook
 *
 * Part of InfoLister by Nickolay Ponomarev <asqueella@gmail.com>
 *  (Thanks to momokatte for his FireSomething extension
 *   and rginda for Venkman's registration code that I learned from).
 * Copyright (c) 2004-2005 Nickolay Ponomarev.
 * See the license.txt included in this package for more information.
 */

const IL_CLASS_ID = Components.ID("{2ee2ad53-be3a-2b41-7a75-12533ba32165}");
const IL_CONTRACT_ID = "@mozilla.org/network/protocol/about;1?what=info";
const nsIAboutModule = Components.interfaces.nsIAboutModule;
const nsISupports = Components.interfaces.nsISupports;
var AboutInfoService = null;

// the implementation component itself
function AboutInfoServiceImpl() {
}
AboutInfoServiceImpl.prototype = {
  newChannel: function(aURI) {
    // load common.js
    var loader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
      getService(Components.interfaces.mozIJSSubScriptLoader);
    loader.loadSubScript("chrome://infolister/content/common.js");

    var ioService = ILHelpers.getService("network/io-service;1", "nsIIOService");
    var il = InfoListerGlue.getInfoLister();

    // generate the information file (f)
    var f = il.makeLocalFile();
    il.saveToFile(f);
    var url = ILHelpers.createInstance("network/protocol;1?name=file",
                "nsIFileProtocolHandler").getURLSpecFromFile(f);

    // get a channel for created file
    var channel = ioService.newChannel(url, null, aURI);
    switch(il.mode) {
      case 0: //text
        channel.contentCharset = ILPrefs.getLocalizedUnicharPref("charset");
        channel.contentType = "text/plain";
        break;
      case 1:
        channel.contentType = "text/html";
        break;
      case 2: //xml
        channel.contentType = "text/xml";
        break;
      default:
        ILHelpers.dump("warning: unknown mode: " + il.mode);
    }
    return channel;
  },
  
  QueryInterface: function(aIID) {
    if(!aIID.equals(nsISupports) && !aIID.equals(nsIAboutModule))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};

const nsIComponentRegistrar = Components.interfaces.nsIComponentRegistrar;
var ILModule = {
  registerSelf: function (aCompMgr, aFileSpec, aLocation, aType) {
    aCompMgr.QueryInterface(nsIComponentRegistrar).
      registerFactoryLocation(IL_CLASS_ID, "InfoLister's About Protocol Handler",
        IL_CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aFileSpec, aLocation) {
    aCompMgr.QueryInterface(nsIComponentRegistrar).
      unregisterFactoryLocation(IL_CLASS_ID, aFileSpec);
  },

  getClassObject: function (aCompMgr, aCID, aIID) {
    if(aCID.equals(IL_CLASS_ID)) return { // the factory
      createInstance: function(aOuter, aIID) {
        if(aOuter)
          throw Components.results.NS_ERROR_NO_AGGREGATION;
        if(!aIID.equals(nsISupports) && !aIID.equals(nsIAboutModule))
          throw Components.results.NS_ERROR_INVALID_ARG;

        if(!AboutInfoService)
          AboutInfoService = new AboutInfoServiceImpl();
        return AboutInfoService;
      }
    };

    if(!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  canUnload: function() {
    return true;
  }
};

function NSGetModule(aCompMgr, aFileSpec) {
  return ILModule;
}

