/* format.js
 * Routines formatting HTML/Text files from the in-memory XML.
 * (The XML can also be saved/used as is, with an XSLT stylesheet.)
 * Requires: common.js, collector.js
 *
 * Part of InfoLister by Nickolay Ponomarev <asqueella@gmail.com>
 * Copyright (c) 2004-2005, Nickolay Ponomarev.
 * See the license.txt included in this package for licensing information.
 */

function ILFormatter() {
}

ILFormatter.prototype = {
  formatInfo: function(doc, templPrefs) {
    var infoElt = doc.getElementsByTagName("info")[0];
    if(InfoLister.mode == 2) { // XML
      var mode = ILPrefs.getIntPref("XSLT.mode");
      var stylesheetURL = ILPrefs.getUnicharPref("XSLT.file");
      //stylesheetURL = "chrome://infolister/content/infolister.xsl";
      
      if(mode == 2) { // process ourselves
        try {
          var processor = new XSLTProcessor();
          
          var testTransform = document.implementation.createDocument("", "test", null);
          //testTransform.addEventListener("load", onload, false);
          testTransform.async = false; // xxx?
          testTransform.load(stylesheetURL);
          processor.importStylesheet(testTransform);
          var result = processor.transformToDocument(doc);
          doc = result; // xxx instead get the text from result when outputmode=text; also set our mode (for about:info f.e.)
        } catch(e) {
          ILHelpers.dumpException(e);
        }
      }

      if(mode == 1) { // add PI
        var pi = doc.createProcessingInstruction("xml-stylesheet",
          'type="text/xsl" href="' + stylesheetURL + '"');
        doc.insertBefore(pi, infoElt);
      }
      return ((new XMLSerializer).serializeToString(doc));
    }

    var html = this.html = InfoLister.html;

    function include(aName) {
      return templPrefs.getBoolPref(aName + "bool");
    }
    function templText(aName) {
      return templPrefs.getUnicharPref(aName + "string");
    }
    
    function makeBox(aClass, aHeader, aList) {
      if(html)
        return '<div class="' + aClass + '">\n<h3>' + aHeader +
               "</h3>\n<ul>\n" + aList + "\n</ul>\n</div>";
      else
        return "*** " + aHeader + "\n" + aList + "\n";
    }
    function makeListItem(aStr) {
      return (html ? "<li>" + aStr + "</li>" : aStr) + "\n";
    }
    function buildAddonStringFromTemplate(aTitle) {
      var val;
      for (var attr in ILInfoCollector.emAttributes) {
        if(attr == "name")
          val = node.firstChild.data;
        else
          val = node.getAttribute(attr);
        aTitle = aTitle.replace(new RegExp("%"+attr+"%","g"), val ? val : "");
      }
      return aTitle;
    }

    var infoText = "";
    var addonStr, header;
    var totalCount, list, templateString;
    var treeWalker = doc.createTreeWalker(infoElt, NodeFilter.SHOW_ELEMENT, null, true);
    var globalVarValues = {};
    var node = treeWalker.firstChild();
    while(node) {
      var nodeName = node.localName;
      if(nodeName == "lastupd")
      {
        globalVarValues.lastupd = node.firstChild.data;
        if(include("lastupd"))
          infoText += this.formatTextData("lastupd", globalVarValues.lastupd);
      }
      else if(nodeName == "useragent")
      {
        globalVarValues.ua = node.firstChild.data;
        if(include("ua"))
          infoText += this.formatTextData("useragent", globalVarValues.ua);
      }
      else if(nodeName == "extensions")
      {
        if(include("extension")) {
        totalCount = 0;
        list = "";
        var disabledCount = 0;
        var disabledList = "";
        node = treeWalker.firstChild(); // != null, as InfoLister is an extension
        templateString = templPrefs.getUnicharPref("extensionstring");
        var oneList = templPrefs.getBoolPref("onelistbool");
        
        while(node) {
          addonStr = buildAddonStringFromTemplate(templateString);
          if(node.getAttribute("disabled") == "true") {
            disabledCount ++;
            if(oneList) { // put disabled and enabled exts in a single list
              list += makeListItem(addonStr + " [" + 
                this.getStr("disabled") + "]");
            } else { // two separate lists
              disabledList += makeListItem(addonStr);
            }
          } else
            list += makeListItem(addonStr);
          totalCount ++;
          node = treeWalker.nextSibling();
        }
        node = treeWalker.parentNode();

        if(!oneList) {
          header = this.makeHeader(this.getStr("enabledextensions")) +
            " (" + totalCount + ")";
          infoText += makeBox("extensions", header, list);

          if(disabledCount > 0) {
            header = this.makeHeader(this.getStr("disabledextensions")) +
              " (" + disabledCount + ")";
            infoText += makeBox("disabledextensions", header, disabledList);
          }
        } else {
          header = this.makeHeader(this.getStr("extensions")) + " (" +
            this.getStr("enabled") + ": " + (totalCount - disabledCount) + ", " +
            this.getStr("disabled") + ": " + disabledCount + ")";
          infoText += makeBox("extensions", header, list);
        }
        }
      }
      else if(nodeName == "themes")
      {
        if(include("theme")) {
        totalCount = 0;
        node = treeWalker.firstChild(); // != null, as InfoLister is an extension
        list = "";
        templateString = templPrefs.getUnicharPref("themestring");

        while(node) {
          addonStr = buildAddonStringFromTemplate(templateString);
          if(node.getAttribute("selected") == "true")
            addonStr += " [" + this.getStr('selected') + "]";
          totalCount ++;
          list += makeListItem(addonStr);
          node = treeWalker.nextSibling();
        }
        node = treeWalker.parentNode();
        header = this.makeHeader(this.getStr("themes")) + " (" + totalCount + ")";
        infoText += makeBox("themes", header, list);
        }
      }
      else if(nodeName == "plugins")
      {
        if(include("plugin")) {
        totalCount = 0; list = "";
        node = treeWalker.firstChild();
        while(node) {
          totalCount ++;
          list += makeListItem(node.firstChild.data);
          node = treeWalker.nextSibling();
        }
        node = treeWalker.parentNode();
        header = this.makeHeader(this.getStr("plugins")) + " (" + totalCount + ")";
        infoText += makeBox("plugins", header, list);
        }
      }
      else {
        ILHelpers.dump("Warning! Unknown element in generated XML: " + nodeName);
      }
      node = treeWalker.nextSibling();
    }

    // ------- Prefix
    var prefix = include("prefix") ? templText("prefix") : "";
    var postfix = include("postfix") ? templText("postfix") : "";
    var head = include("head") ? templText("head") : "";
    var globVars = ["ua", "lastupd"];

    for (var i in globVars) { // process data in prefix
      var variable = globVars[i];
      var value = globalVarValues[globVars[i]]; // xxx
      prefix = prefix.replace(new RegExp("%"+variable+"%","g"), value);
      postfix = postfix.replace(new RegExp("%"+variable+"%","g"), value);
      head = head.replace(new RegExp("%"+variable+"%","g"), value);
    }

    // ------- Final
    if(html)
    {
      var charset = ILPrefs.getLocalizedUnicharPref("charset");
      head = '\n<meta http-equiv="Content-Type" content="text/html; charset=' +
        charset + '">\n' + head;
      head = (head == "") ? "" : "<head>" + head + "</head>\n";
      var doctype = '<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">\n';

      return doctype + "<html>" + head + "\n<body>\n" +
             prefix + infoText + postfix + "\n</body>\n</html>";
    }
    return prefix + infoText + postfix;
  },

  // change '\n' line-breaks used in DOM to platform-specific line-breaks
  fixLineBreaks: function(aText) {
    var platform = window.navigator.platform.toLowerCase();
    if(platform.indexOf("win") > -1)
      return aText.replace(/\n/mg, "\r\n");
    if(platform.indexOf("mac") > -1)
      return aText.replace(/\n/mg, "\r");
    return aText;
  },

  makeHeader: function(aHeader) {
    if(this.html)
      return '<span class="hdr">' + aHeader + '</span>';
    return aHeader;
  },

  formatTextData: function(aTitle, aData) {
    var text = "";
    if(this.html) text += '<div class="' + aTitle + '">\n';
    text += this.makeHeader(this.getStr(aTitle)) +
                " " + aData + "\n";
    if(this.html) text += "</div>";
    return text + "\n";
  },
  
  
  // l10n helpers
  getStringBundle: function() {
    if(!this.stringBundle) {
      var sbSvc = ILHelpers.getService("intl/stringbundle;1", "nsIStringBundleService");
      this.stringBundle = sbSvc.createBundle("chrome://infolister/locale/infolister.properties");
    }
    return this.stringBundle;
  },

  getStr: function(s) {
    return this.getStringBundle().GetStringFromName(s);
  }
};

