/* prefs.js
 * Defines: preferences object used in Settings window.
 * Requires: common.js
 *
 * Part of InfoLister by Nickolay Ponomarev <asqueella@gmail.com>
 * Copyright (c) 2004-2005, Nickolay Ponomarev.
 * FTP Settings part is based on Torisugari's BookmarksFTP code.
 * See the license.txt included in this package for licensing information.
 */

const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var gStringBundle;
var gDeck;

const PREFS = {
  Text: {
    prefix:    { tooltip: "global-vars" },
    theme:     { tooltip: "ext-vars" },
    extension: { tooltip: "ext-vars" },
    postfix:   { tooltip: "global-vars" }
  },
  HTML: {
    head:      { tooltip: "global-vars" },
    prefix:    { tooltip: "global-vars" },
    theme:     { tooltip: "ext-vars" },
    extension: { tooltip: "ext-vars" },
    postfix:   { tooltip: "global-vars" }
  },
  XML: {
    theme:     {},
    extension: {}
  }
}

const BOOL_PREFS = {
  lastupd: "", ua: "", plugin: "", onelist: ""
}


const MODES = {
  Text: 0,
  HTML: 1,
  XML: 2
}

function el(aID) {
  return document.getElementById(aID);
}


var DialogBuilder = {
  // create a row with checkbox and a textbox for pref aPref
  _createRow: function(aPref, aMode) {
    var row = document.createElementNS(XUL_NS, "row");

    var cb = document.createElementNS(XUL_NS, "checkbox");
    cb.id = aMode + "." + aPref + "bool";
    cb.setAttribute("label", getStr(aPref));
    row.appendChild(cb);

    var t = document.createElementNS(XUL_NS, "textbox");
    t.id = aMode + "." + aPref + "string";
    t.setAttribute("flex", "1");
    t.setAttribute("multiline", "true");
    row.appendChild(t);
    return row;
  },
  
  // fill given grid with rows from TEMPL_PREFS
  fillBox: function(aBox, aMode) {
    var grid = aBox.firstChild;
    var rows = document.createElementNS(XUL_NS, "rows");
    grid.appendChild(rows);
    
    var prefs = PREFS[aMode];
    for(var i in prefs) {
      var row = this._createRow(i, aMode);
      row.setAttribute("tooltip", prefs[i].tooltip);
      rows.appendChild(row);
    }
    
    var hbox = grid.nextSibling; // the box that contains lastupd, ua and plugins checkboxes
    hbox.childNodes[0].id = aMode + ".lastupdbool";
    hbox.childNodes[1].id = aMode + ".uabool";
    hbox.childNodes[2].id = aMode + ".pluginbool";
    hbox.nextSibling.id = aMode + ".onelistbool";
  },
  
  fillXMLBox: function(aBox) {
    var prefs = { ua: "", lastupd: "",
      // these use special strings from the .properties file:
      extension: "XML.", theme: "XML.", plugin: "XML." };
      
    for(var i in prefs) {
      var cb = document.createElementNS(XUL_NS, "checkbox");
      cb.id = "XML." + i + "bool";
      aBox.appendChild(cb);
      cb.label = getStr(prefs[i] + i);
    }
  },
  
  // build the dialog
  build: function() {
    var boxText = gDeck.firstChild;
    var boxHTML = boxText.cloneNode(true);
    var boxXML = el("XML");
    gDeck.appendChild(boxHTML);
    gDeck.appendChild(boxXML); // make it last (3rd) child
    DialogBuilder.fillBox(boxHTML, "HTML");
    DialogBuilder.fillBox(boxText, "Text");
    DialogBuilder.fillXMLBox(boxXML.firstChild);
    sizeToContent();
  }
}

var InfoListerPrefsDialog = {
  onLoad: function() {
    gStringBundle = el("strbundle");
    gDeck = el("modeSpecific");

    DialogBuilder.build();

    // persist selected tab - workaround bug 256024, part 1
    var selectedIndex = el("dialog").getAttribute("seltab");
    if(selectedIndex)
      el("tabbox").selectedIndex = selectedIndex;

    this.readPrefs();
    FTPPrefsDialog.doLoad();
  },
  
  onUnload: function() {
    // persist selected tab - workaround bug 256024, part 2
    el("dialog").setAttribute("seltab", el("tabbox").selectedIndex);
  },

  onAccept: function() {
    this.writePrefs();
    FTPPrefsDialog.flush();

    var infowin = InfoListerWindows.get(InfoListerWindows.Info);
    if(infowin)
    { // update infolister.xul that opened us
      try {
        var il = infowin.InfoLister;
        var file = il.makeLocalFile();
        var data = il.generateInfo();
        il.saveToFile(file, data);
        il.printInfo(data);
      } catch(e) {
        ILHelpers.dumpException(e);
      }
    }
    return true;
  },


  readPrefs: function() {
    el("outputmode").selectedIndex = ILPrefs.getIntPref("ouputmode");
    el("XSLT.mode").selectedIndex = ILPrefs.getIntPref("XSLT.mode");
    el("XSLT.file").value = ILPrefs.getUnicharPref("XSLT.file");
    OutputFilePref.read();
    this.readwriteTemplatePrefs(true);
    this.modeChanged();//?
  },
  
  writePrefs: function() {
    ILPrefs.setIntPref("ouputmode", el("outputmode").selectedIndex);
    ILPrefs.setIntPref("XSLT.mode", el("XSLT.mode").selectedIndex);
    ILPrefs.setUnicharPref("XSLT.file", el("XSLT.file").value);
    OutputFilePref.write();
    this.readwriteTemplatePrefs(false);
  },

  // reads or writes customizable template strings (depending on aRead)
  readwriteTemplatePrefs: function(aRead) {
    var i, id, textbox, checkbox;
    for(var mode in MODES) {
      var prefs = PREFS[mode];
      for(i in prefs) {
        id = mode + "." + i + "bool";
        checkbox = document.getElementById(id);
        if(aRead)
          checkbox.checked = ILPrefs.getBoolPref(id);
        else
          ILPrefs.setBoolPref(id, checkbox.checked);
        if(mode == "XML") continue;
        id = mode + "." + i + "string";
        textbox = document.getElementById(id);
        if(aRead)
          textbox.value = ILPrefs.getUnicharPref(id);
        else
          ILPrefs.setUnicharPref(id, textbox.value);
      }
      for(i in BOOL_PREFS) {
        id = mode + "." + i + "bool";
        checkbox = document.getElementById(id);
        if(checkbox) {
          if(aRead)
            checkbox.checked = ILPrefs.getBoolPref(id);
          else
            ILPrefs.setBoolPref(id, checkbox.checked);
        }
      }
    }
  },

  // process Mode radiogroup click
  modeChanged: function() {
    var newmode = el("outputmode").selectedIndex;
    gDeck.selectedIndex = newmode;
  }
};


// responsible for managing the output file pref
var OutputFilePref = {
  _outputFile: null,
  valid: true,
  getOutputFile: function() {
    var path = el("filename").value;
    this.valid = true;
    if(!this._outputFile || (this._outputFile.path != path))  // xxx?
    {
      if(path != "") {
        try {
          this._outputFile = Components.classes["@mozilla.org/file/local;1"].
                             createInstance(Components.interfaces.nsILocalFile);
          this._outputFile.initWithPath(path);
        } catch(e) {
          this._outputFile = null;
          this.valid = false;
        }
      } else
        this._outputFile = null;
    }
    return this._outputFile;
  },

  checkValid: function() {
    this.getOutputFile(); // this sets |valid|
    if(!this.valid) {
      alert(gStringBundle.getFormattedString("prefs.invalidpath", 
        [el("filename").value]));
      return false;
    }
    return true;
  },

  // display a filepicker to set the output file
  browseForFile: function() {
    var file = this.getOutputFile();
    var rv = ILHelpers.pickFile("", file);
    if(!rv) return; // Cancel

    this._outputFile = rv.file;
    el("filename").value = rv.file.path;
  },
  
  // write info to specified file
  rewriteFile: function() {
    if(!this.checkValid()) return;
    var apply = confirm(getStr("applyprompt"));
    if(!apply) return;

    InfoListerPrefsDialog.onAccept();

    var il = InfoListerGlue.getInfoLister(window);

    try {
      var file = il.makeLocalFile();
      il.saveToFile(file);
    } catch(e) {
      alert("Couldn't write to the file: \n\n" + e);
    }
  },
  
  // read pref
  read: function() {
    try {
      this._outputFile = ILPrefs.getFilePref("file");
      el("filename").value = this._outputFile.path;
    } catch(e) {
    }
  },
  // write pref
  write: function() {
    var file = OutputFilePref.getOutputFile(); // also sets |valid|
    if(OutputFilePref.checkValid()) { // do not touch the file pref if new value is invalid
      if(file)
        ILPrefs.setFilePref("file", file);
      else
        ILPrefs.clearUserPrefSafe("file");
    }
  }
};

// FTP Settings panel implementation
var gFtpBranch = ILPrefs.getSubBranch("ftp.");
var ftpPrefs = { host: "", username: "", path: "", protocol: "" };

var FTPPrefsDialog = {
  flush: function()
  {
    gFtpBranch.setBoolPref("autoupload", el("autoupload").checked);
    var password = el("password").value;
    for(var i in ftpPrefs) {
      ftpPrefs[i] = el(i).value;
      if(ftpPrefs[i])
        gFtpBranch.setUnicharPref(i, ftpPrefs[i]);
      else { // reset to default value [needed for protocol] [xxx?]
        gFtpBranch.clearUserPrefSafe(i);
        ftpPrefs[i] = gFtpBranch.getUnicharPref(i);
      }
    }

    if(ftpPrefs.host || ftpPrefs.username) {
      var pm = ILHelpers.createInstance("passwordmanager;1", "nsIPasswordManager");
      try {
        pm.removeUser(ftpPrefs.host, ftpPrefs.username);
      } catch(e) {}
      pm.addUser(ftpPrefs.host, ftpPrefs.username, password);
    }
  },

  doLoad: function()
  {
    el("autoupload").checked = gFtpBranch.getBoolPref("autoupload");
    for(var i in ftpPrefs)
      ftpPrefs[i] = gFtpBranch.getUnicharPref(i);

    var password = { value: "" };
    if(ftpPrefs.host || ftpPrefs.username)
    {
      var host = { value: "" };
      var username = { value: "" };

      try {
        var pmi = ILHelpers.createInstance("passwordmanager;1", "nsIPasswordManagerInternal");
        pmi.findPasswordEntry(ftpPrefs.host, ftpPrefs.username, "",
                              host, username, password);
        ftpPrefs.host = host.value;
        ftpPrefs.username = username.value;
      } catch(e) {}
    }
    el("password").value = password.value;

    for(i in ftpPrefs) {
      if(i != "protocol")
        el(i).value = ftpPrefs[i];
      else
        el(i).selectedItem = el(ftpPrefs.protocol);
    }
    this.updateUploadDisabledState();
  },

  // Upload button handler
  updateUploadDisabledState: function() {
    el("upload").disabled = 
      (el("host").value == "") ||
      (el("path").value == "");
  },

  upload: function() {
    refresh(); // xxx
    var il = InfoListerGlue.getInfoLister(window);
    il.uploadInfo();
  }
}

//xxx
function refresh()
{
  FTPPrefsDialog.flush();
  FTPPrefsDialog.doLoad();
}

function getStr(aStr) {
  try {
    return gStringBundle.getString(aStr);
  } catch(e) {
    throw "Failure while getting string '" + aStr + "' from locale: \n" + e;
  }
}

